// project created on 12/16/2005 at 9:29 PM
using System;
using System.IO;
using System.Net;
using System.Xml;
using Gtk;
using Gnome.Vfs;
using DBus;

[Interface("org.gnome.Banshee.Core")]
public abstract class BansheeCore
{       
    public static BansheeCore FindInstance()
    {
        Connection connection = Bus.GetSessionBus();
        Service service = Service.Get(connection, "org.gnome.Banshee");        
        return (BansheeCore)service.GetObject(typeof(BansheeCore), "/org/gnome/Banshee/Core");
    }

    [Method] public abstract void ImportFiles(String[] Uris);
}

public class GladeApp
{

	Window podcast_window;
	HBox h;
	VBox v;
	
	Label label1;
	Label podcast;
	Entry podcast_entry;
	Button add_podcast_button;
	Button update_podcast;
	ProgressBar vooruitgang;
	ListStore active_podcasts;
	TreeView overzicht;
	ScrolledWindow sw;
	string file;
	string path;
	private static BansheeCore banshee = null;
		
	public static void Main (string[] args)
	{
		try {
			banshee = BansheeCore.FindInstance();
        	} catch(Exception) {
        		Console.Error.WriteLine("Could not locate Banshee on D-Bus. Perhaps it's not running?");
            		Environment.Exit(1);
        	}
		new GladeApp (args);
	}

	public GladeApp (string[] args) 
	{
		Gtk.Application.Init ();
		h = new HBox();
		v = new VBox();
		podcast_window = new Window("Banshee - Podcast");
		podcast_entry = new Entry("/home/thijs/temp.xml");
		add_podcast_button = new Button("Add");
		update_podcast = new Button("Update All");
		label1 = new Label("Nieuw :");
		podcast = new Label("processing ...");
		active_podcasts = new ListStore(typeof(String),typeof(String),typeof(String),typeof(String));
		overzicht = new TreeView();
		overzicht.Model = active_podcasts;
		vooruitgang = new ProgressBar();
		sw = new ScrolledWindow();
		file = "/home/thijs/temp";
		path = "/home/thijs/";
		Gnome.Vfs.Vfs.Initialize();
		sw.Add(overzicht);
		sw.Show();
		overzicht.AppendColumn ("Status", new CellRendererText (),"text", 0);
		overzicht.AppendColumn ("Naam", new CellRendererText (), "text", 1);
		overzicht.AppendColumn ("Last podcast", new CellRendererText (), "text", 2);
		overzicht.AppendColumn ("source",new CellRendererText(), "text", 3);
		// add widgets
		h.PackStart(label1, false , false, 0);
		h.PackStart(podcast_entry, false, false, 0);
		h.PackStart(add_podcast_button, false, false, 0);
		h.PackStart(update_podcast, false, false, 0);
		v.PackStart(h, false, false, 0);
		v.PackStart(sw, false, false, 0);
		v.PackStart(vooruitgang, false, false, 0);
		v.PackStart(podcast, false, false, 0);	
		podcast_window.Add(v);
		overzicht.HeadersVisible = true;
		// set up event handling
		podcast_window.DeleteEvent += new DeleteEventHandler(podcastDelete);
		add_podcast_button.Clicked += addPodcastClicked;		
		update_podcast.Clicked += updateAllClicked;
		podcast_window.ShowAll();
		Gtk.Application.Run ();
	}
	
	private void podcastDelete(object o, DeleteEventArgs args){
		Gnome.Vfs.Vfs.Shutdown();
		Gtk.Application.Quit();
	}
	
	private void addPodcastClicked(object o, EventArgs args){
		active_podcasts.AppendValues("Just Added","New Podcast","--",podcast_entry.Text);
		//Console.WriteLine("Der is op de knop gedrukt\n");
		// Temp use an easeair way
		//GetPodcast(new Uri(podcast_entry.Text));
		//GetPodcast(new Uri("http://www.soundtrack.net/rss/podcast.rss"));
		//GetTittlePodcast();
		//vooruitgang.Pulse();
	}

	private void updateAllClicked(object o, EventArgs args)
	{
	    TreeIter iter = new TreeIter ();
	    Boolean gelukt = true;
	    int counter = 1;
	    Console.WriteLine("Updating all podcasts");
	//    foreach (object[] row in active_podcasts)
	    gelukt = active_podcasts.GetIterFirst(out iter);
	    while (gelukt){
		// Download the podcast to the HD
		active_podcasts.SetValue(iter,0,"updating");
		vooruitgang.Pulse();
		Console.WriteLine(" Downloading : {0}",(String) active_podcasts.GetValue(iter,3));
		GetPodcast(new System.Uri((String) active_podcasts.GetValue(iter,3)),counter);
		podcast.Text = "Searching for name";
		vooruitgang.Pulse();
		// Update List
		// Search for the name of the podcast
		active_podcasts.SetValue(iter,1,GetNameOfPodcast(counter));
		podcast.Text = "Searching for podcast";
		vooruitgang.Pulse();
		// Update List
		// Search for latest podcast
		active_podcasts.SetValue(iter,2,GetLatestPodcast(counter));
		podcast.Text = "Downloading podcast";
		vooruitgang.Pulse();
		// Update List
		// Look if latest podcast is already downloaded
		// Remove old podcast from drive
		// update list
		// remove old podcast from banshee
		//  update list
		// download new podcast
		DownloadPodcast(new Gnome.Vfs.Uri((String)active_podcasts.GetValue(iter,2)),counter);	
		podcast.Text = "Adding to Banshee";
		vooruitgang.Pulse();
		// add new podcast to banshee
		banshee.ImportFiles((path+"p"+counter+"_"+(new Gnome.Vfs.Uri((String)active_podcasts.GetValue(iter,2))).ExtractShortName()).Split((";").ToCharArray(),1));
		// update list
		podcast.Text = "Done";
		active_podcasts.SetValue(iter,0,"Done");
		gelukt = active_podcasts.IterNext(ref iter);
		counter++;
	    }
	}
		

	public void GetPodcast(System.Uri podcastUri, int count){
		// Initialize podcastUri
		WebRequest request = WebRequest.Create(podcastUri);
		WebResponse response = request.GetResponse();
		// Make the tempFile
		using (TextWriter writer = System.IO.File.CreateText(file+count+".xml")){
			using (StreamReader reader = 
				new StreamReader(response.GetResponseStream()))
			{
				writer.WriteLine(reader.ReadToEnd());
			}
		}
	}

	public String GetNameOfPodcast(int count)
	{
		String test = file+count+".xml"; 
		XmlDocument document = new XmlDocument();
		int aantal = 0;
		document.Load(test);	
		XmlNodeList channels = document.DocumentElement.SelectNodes("channel");
		XmlNodeList titles;
		String Title = "";
		foreach(XmlNode channel in channels){
			titles = channel.SelectNodes("title");
			foreach(XmlNode title in titles){
				Title = title.InnerText;
			}
		}
		return Title;
	}

	private String GetLatestPodcast(int count)
	{
		String test = file + count + ".xml";
		XmlDocument document = new XmlDocument();
		int aantal = 0;
		document.Load(test);
		XmlNodeList channels = document.DocumentElement.SelectNodes("channel");
		XmlNodeList items;
		String first = "";
		foreach(XmlNode channel in channels){
			items = channel.SelectNodes("item");
			foreach(XmlNode item in items){
				if (first.Equals("")) first = item.SelectSingleNode("guid").InnerText;
			}
		}
		return first;	

	}

	private void DownloadPodcast(Gnome.Vfs.Uri link,int count)
	{
		Gnome.Vfs.Result result = Gnome.Vfs.Result.Ok;
		Gnome.Vfs.Uri source = link;
		Gnome.Vfs.Uri target = new Gnome.Vfs.Uri (path+"p"+count+"_"+link.ExtractShortName());
		Gnome.Vfs.XferProgressCallback cb = new Gnome.Vfs.XferProgressCallback (Progress);
		result = Gnome.Vfs.Xfer.XferUri (source, target, 
						     Gnome.Vfs.XferOptions.Default, 
						     Gnome.Vfs.XferErrorMode.Abort, 
						     Gnome.Vfs.XferOverwriteMode.Replace, 
						     cb);
		Console.WriteLine (path+"p"+count+"_"+link.ExtractShortName());
	}
	
	private int Progress (Gnome.Vfs.XferProgressInfo info)
	{
		if (info.BytesTotal > 0) {
			//podcast.Text = (String)(info.BytesCopied / (double)info.BytesTotal);
		}
		return 1;
	}

	public void GetTittlePodcast(){
		XmlDocument document = new XmlDocument();
		int aantal = 0;
		document.Load(file);
		
		XmlNodeList channels = document.DocumentElement.SelectNodes("channel");
		XmlNodeList titles;
		XmlNodeList links;
		XmlNodeList items;
		String first = "";
		String Title = "";
		Console.WriteLine("Er zijn {0} kanalen in deze podcast !! ",channels.Count);
		foreach(XmlNode channel in channels){
			Console.WriteLine(" Kanaal nummer {0}:",++aantal);
			titles = channel.SelectNodes("title");
			Console.WriteLine("Er zijn {0} titles in deze podcast !!",titles.Count);
			foreach(XmlNode title in titles){
				Console.WriteLine(" Title : {0}",title.InnerText);
				Title = title.InnerText;
			}
			links = channel.SelectNodes("link");
			foreach(XmlNode link in links){
				Console.WriteLine(" Link : {0}",link.InnerText);
			}
			items = channel.SelectNodes("item");
			Console.WriteLine("Er zijn {0} items in deze podcast",items.Count);
			active_podcasts.Clear();
			foreach(XmlNode item in items){
				if (first.Equals("")) podcast.Text = item.SelectSingleNode("guid").InnerText;
				if (first.Equals("")) first = "-";
				active_podcasts.AppendValues("",Title,item.SelectSingleNode("guid").InnerText);
				Console.WriteLine(item.SelectSingleNode("guid").InnerText);
			}
		}
	}
}
